''
'' This code is part of https://developer.mescius.com/document-solutions/dot-net-imaging-api/demos.
'' Copyright (c) MESCIUS inc. All rights reserved.
''
Imports System
Imports System.IO
Imports System.Drawing
Imports System.Reflection
Imports System.Collections.Generic
Imports System.Diagnostics
Imports GrapeCity.Documents.Imaging
Imports GrapeCity.Documents.Drawing
''
'' This program creates an instance of the AllBlendingModes class and uses it to generate an image file.
'' The GrapeCity packages needed to build and run this sample are available on NuGet.org.
''
'' Command line switches:
'' /y, -y -- assume all defaults, overwrite existing files.
'' /q, -q -- quiet operation (ignored unless /y is specified).
'' /n, -n -- generate repeatable 'random' data.
'' /f path, -f path -- use 'name' as base file path/name.
''
Module Program
    Sub Main(args As String())
        Dim yes2all = false
        Dim quiet = false
        Dim norandom = False
        Dim outMs As MemoryStream = Nothing
        Dim outTw As TextWriter = Nothing
        Try
            Dim fname = "AllBlendingModes"
            For i = 0 to args.Length - 1
                Select Case args(i)
                    Case "/y", "-y"
                        yes2all = True
                    Case "/q", "-q"
                        quiet = True
                    Case "/n", "-n"
                        norandom = True
                    Case "/f", "-f"
                        If i + 1 < args.Length Then
                            i += 1
                            If Directory.Exists(args(i)) Then
                                fname = Path.Combine(args(i), fname)
                            Else
                                fname = args(i)
                            End If
                        End If
                End Select
            Next i
            If norandom Then
                Util.NoRandom = True
            End If
            If Not yes2all Then
                quiet = False
            End If
            If quiet Then
                outMs = New MemoryStream()
                outTw = New StreamWriter(outMs)
                Console.SetOut(outTw)
            End If

            Dim lk = Environment.GetEnvironmentVariable("DOCSOL_LKEY")
            if Not String.IsNullOrEmpty(lk) Then
                GcBitmap.SetLicenseKey(lk)
            End If

            '' Find out which image creating method this sample supports, and what is its default image encoding:
            Dim sampleType as Type = GetType(DsImagingWeb.Demos.AllBlendingModes)
            Dim generateStream = sampleType.GetMethod("GenerateImageStream", { GetType(String), GetType(Size), GetType(Single), GetType(Boolean), GetType(String()) })
            Dim generateImage = sampleType.GetMethod("GenerateImage", { GetType(Size), GetType(Single), GetType(Boolean), GetType(String()) })
            Dim defaultMime = sampleType.GetProperty("DefaultMime")
            '' Optional sample parameters:
            Dim sampleParams = Nothing
            Dim getParamsList = sampleType.GetMethod("GetSampleParamsList", BindingFlags.Static Or BindingFlags.Public)
            If getParamsList IsNot Nothing Then
                Dim pars = TryCast(getParamsList.Invoke(Nothing, Nothing), List(Of String()))
                If pars IsNot Nothing Then
                    sampleParams = pars(-1)
                End If
            End If

            Dim sample = New DsImagingWeb.Demos.AllBlendingModes()
            '' Adjust the target file name according to the image encoding:
            Dim imageMime = Util.MimeTypes.JPEG
            if defaultMime IsNot Nothing Then
                imageMime = CType(defaultMime.GetValue(sample), String)
            End If
            Dim ext = ""
            Select Case imageMime
                Case MimeTypes.JPEG
                    ext = ".jpeg"
                Case MimeTypes.PNG
                    ext = ".png"
                Case MimeTypes.TIFF
                    ext = ".tiff"
                Case MimeTypes.GIF
                    ext = ".gif"
                Case MimeTypes.BMP
                    ext = ".bmp"
                Case MimeTypes.ICO
                    ext = ".ico"
                Case MimeTypes.SVG
                    ext = ".svg"
                Case Else
                    System.Diagnostics.Debug.Assert(false)
            End Select
            fname = Path.ChangeExtension(fname, ext)

            If Not yes2all Then
                Console.WriteLine($"Press ENTER to create '{fname}' or enter an alternative file name:")
                Dim t = Console.ReadLine()
                If Not string.IsNullOrWhiteSpace(t) Then
                    fname = t
                End If
                fname = Path.GetFullPath(fname)
                If File.Exists(fname) Then
                    Console.WriteLine($"'{fname}' exists." + vbCrLf + "OVERWRITE (Y/n)?")
                    Dim yesno = Console.ReadLine()
                    If yesno.ToLower() = "n" Then
                        Console.WriteLine("Aborted.")
                        Return
                    End If
                End If
            End If

            '' Generate the image using the method available for this sample:
            If Not quiet Then
                Console.WriteLine($"Generating '{fname}'...")
            End If
            If generateStream IsNot Nothing Then
                Dim stream = CType(generateStream.Invoke(sample, New Object() { imageMime, New Size(1024, 1024), 96, True, sampleParams }), Stream)
                stream.Seek(0, SeekOrigin.Begin)
                Using fs = File.Create(fname)
                    stream.CopyTo(fs)
                End Using
            Else
                Dim bmp = CType(generateImage.Invoke(sample, New Object() { New Size(1024, 1024), 96, True, sampleParams }), GcBitmap)

                Select Case imageMime
                    Case MimeTypes.JPEG
                        bmp.SaveAsJpeg(fname, Nothing, 70)
                    Case MimeTypes.PNG
                        bmp.SaveAsPng(fname)
                    Case MimeTypes.TIFF
                        bmp.SaveAsTiff(fname)
                    Case MimeTypes.GIF
                        bmp.SaveAsGif(fname)
                    Case MimeTypes.BMP
                        bmp.SaveAsBmp(fname)
                    Case MimeTypes.ICO
                        bmp.SaveAsIco(fname, Nothing, IcoFrameEncoding.Png)
                    Case Else
                        System.Diagnostics.Debug.Assert(false)
                End Select
            End If
            If Not quiet Then
                Console.WriteLine($"Created '{fname}' successfully.")
            End If
            '' Uncomment to open the generated file:
            '' ProcessStartInfo psi = new ProcessStartInfo() { FileName = fname, UseShellExecute = true };
            '' Process.Start(psi);
        Catch ex As Exception
            Console.Error.WriteLine($"Error: {ex.Message}")
            If ex.InnerException IsNot Nothing Then
                Console.Error.WriteLine($"InnerException: {ex.InnerException.Message}")
            End If
        Finally
            If Not yes2all Then
                Console.WriteLine("Press any key to exit...")
                Console.ReadKey()
            End If
            If quiet Then
                outTw.Dispose()
                outMs.Dispose()
            End If
        End Try
    End Sub
End Module
